@@ -7,7 +7,7 @@ use warnings;
 use Scalar::Util 'blessed';
 use Carp         'confess';
 
-our $VERSION   = '0.02';
+our $VERSION   = '0.03';
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Class::MOP::Object';
@@ -29,7 +29,14 @@ sub initialize {
     no strict 'refs';
     return bless { 
         '$:package'   => $package_name,
-        '%:namespace' => \%{$package_name . '::'},
+        # NOTE:
+        # because of issues with the Perl API 
+        # to the typeglob in some versions, we 
+        # need to just always grab a new 
+        # reference to the hash in the accessor. 
+        # Ideally we could just store a ref and 
+        # it would Just Work, but oh well :\
+        #'%:namespace' => \%{$package_name . '::'},
     } => $class;
 }
 
@@ -40,7 +47,17 @@ sub initialize {
 # away in the Class::MOP bootstrap section
 
 sub name      { $_[0]->{'$:package'}   }
-sub namespace { $_[0]->{'%:namespace'} }
+sub namespace { 
+    # NOTE:
+    # because of issues with the Perl API 
+    # to the typeglob in some versions, we 
+    # need to just always grab a new 
+    # reference to the hash here. Ideally 
+    # we could just store a ref and it would
+    # Just Work, but oh well :\    
+    no strict 'refs';    
+    \%{$_[0]->name . '::'} 
+}
 
 # utility methods
 
@@ -80,8 +97,8 @@ sub add_package_symbol {
     my ($name, $sigil, $type) = $self->_deconstruct_variable_name($variable); 
 
     no strict 'refs';
-    no warnings 'redefine', 'misc';
-    *{$self->name . '::' . $name} = ref $initial_value ? $initial_value : \$initial_value;    
+    no warnings 'redefine', 'misc';    
+    *{$self->name . '::' . $name} = ref $initial_value ? $initial_value : \$initial_value;      
 }
 
 sub remove_package_glob {
@@ -96,7 +113,7 @@ sub has_package_symbol {
     my ($self, $variable) = @_;
 
     my ($name, $sigil, $type) = $self->_deconstruct_variable_name($variable); 
-
+    
     return 0 unless exists $self->namespace->{$name};   
     
     # FIXME:
@@ -110,10 +127,10 @@ sub has_package_symbol {
     
     if ($type eq 'SCALAR') {    
         my $val = *{$self->namespace->{$name}}{$type};
-        defined $$val ? 1 : 0;        
+        defined(${$val}) ? 1 : 0;        
     }
     else {
-        defined *{$self->namespace->{$name}}{$type} ? 1 : 0;
+        defined(*{$self->namespace->{$name}}{$type}) ? 1 : 0;
     }
 }
 
@@ -194,24 +211,51 @@ Class::MOP::Package - Package Meta Object
 
 =item B<meta>
 
-=item B<initialize>
+=item B<initialize ($package_name)>
 
 =item B<name>
 
+This is a read-only attribute which returns the package name for the 
+given instance.
+
 =item B<namespace>
 
-=item B<add_package_symbol>
+This returns a HASH reference to the symbol table. The keys of the 
+HASH are the symbol names, and the values are typeglob references.
 
-=item B<get_package_symbol>
+=item B<add_package_symbol ($variable_name, ?$initial_value)>
 
-=item B<has_package_symbol>
+Given a C<$variable_name>, which must contain a leading sigil, this 
+method will create that variable within the package which houses the 
+class. It also takes an optional C<$initial_value>, which must be a 
+reference of the same type as the sigil of the C<$variable_name> 
+implies.
 
-=item B<remove_package_symbol>
+=item B<get_package_symbol ($variable_name)>
 
-=item B<remove_package_glob>
+This will return a reference to the package variable in 
+C<$variable_name>. 
+
+=item B<has_package_symbol ($variable_name)>
+
+Returns true (C<1>) if there is a package variable defined for 
+C<$variable_name>, and false (C<0>) otherwise.
+
+=item B<remove_package_symbol ($variable_name)>
+
+This will attempt to remove the package variable at C<$variable_name>.
+
+=item B<remove_package_glob ($glob_name)>
+
+This will attempt to remove the entire typeglob associated with 
+C<$glob_name> from the package. 
 
 =item B<list_all_package_symbols>
 
+This will list all the glob names associated with the current package. 
+By inspecting the globs returned you can discern all the variables in 
+the package.
+
 =back
 
 =head1 AUTHORS
